#include <apr_general.h>
#include <apr_pools.h>
#include <apr_time.h>
#include <apr_thread_proc.h>
#include <apr_file_io.h>

#define APR_WANT_STDIO
#define APR_WANT_STRFUNC
#include <apr_want.h>

#include "svn_types.h"
#include "svn_pools.h"
#include "svn_error.h"
#include "svn_path.h"
#include "svn_repos.h"
#include "svn_fs.h"
#include "svn_repos.h"
#include "svn_time.h"
#include "svn_utf.h"
#include "svn_subst.h"
#include "svn_opt.h"
#include "svn_props.h"
#include "svn_diff.h"
#include "svn_config.h"
#include "config_impl.h"

typedef struct RSSFeed_ctxt_t
{
    svn_repos_t *repos;
    svn_fs_t *fs;
    svn_revnum_t rev_num;
    const char *repos_path;
    const char *config_path;
    svn_fs_txn_t *txn;
    const char *txn_name /* UTF-8! */;
    char *outputbuffer;
    int outputbufferlength;
} RSSFeed_ctxt_t;

typedef struct RSSFeed_config_t
{
    const char * location;
    const char * title;
    const char * link;
    const char * description;
    const char * ttl;
    const char * imageurl;
    const char * difffolder;
    const char * difffolderurl;
    int items;
    int maxsize;
} RSSFeed_config_t;

static svn_error_t *
get_property (svn_string_t **prop_value /* native */,
              RSSFeed_ctxt_t *c, 
              const char *prop_name /* UTF-8! */,
              apr_pool_t *pool)
{
    svn_string_t *raw_value;

    SVN_ERR (svn_fs_revision_prop (&raw_value, c->fs, c->rev_num,
        prop_name, pool));

    if (svn_prop_needs_translation (prop_name))
        SVN_ERR (svn_subst_detranslate_string (&raw_value, raw_value, pool));

    *prop_value = raw_value;

    return SVN_NO_ERROR;
}

/* Generate a diff label for PATH in ROOT, allocating in POOL. */
static svn_error_t *
generate_label (const char **label,
                svn_fs_root_t *root,
                const char *path,
                apr_pool_t *pool)
{
    svn_fs_t *fs = svn_fs_root_fs (root);
    svn_string_t *date;
    const char *datestr;
    const char *name = NULL;
    svn_revnum_t rev = SVN_INVALID_REVNUM;

    if (svn_fs_is_revision_root (root))
    {
        rev = svn_fs_root_revision (root);
        SVN_ERR (svn_fs_revision_prop (&date, fs, rev, 
            SVN_PROP_REVISION_DATE, pool));
    }
    else 
    {
        svn_fs_txn_t *txn;
        name = svn_fs_txn_root_name (root, pool);
        SVN_ERR (svn_fs_open_txn (&txn, fs, name, pool));
        SVN_ERR (svn_fs_txn_prop (&date, txn, SVN_PROP_REVISION_DATE, pool));
    }

    if (date)
    {
        datestr = date->data;
        ((char *)datestr)[10] = ' ';
        ((char *)datestr)[19] = '\0';
    }
    else
    {
        datestr = "                      ";
    }

    if (name)
        *label = apr_psprintf (pool, "%s\t%s UTC (txn %s)", 
        path, datestr, name); 
    else
        *label = apr_psprintf (pool, "%s\t%s UTC (rev %" SVN_REVNUM_T_FMT ")",
        path, datestr, rev);
    return SVN_NO_ERROR;
}

static svn_error_t *
dump_contents (apr_file_t *fh,
               svn_fs_root_t *root,
               const char *path /* UTF-8! */,
               apr_pool_t *pool)
{
    apr_status_t apr_err;
    apr_size_t len, len2;
    svn_stream_t *stream;
    char buffer[1024];

    /* Get a stream to the current file's contents. */
    SVN_ERR (svn_fs_file_contents (&stream, root, path, pool));

    /* Now, route that data into our temporary file. */
    while (1)
    {
        len = sizeof (buffer);
        SVN_ERR (svn_stream_read (stream, buffer, &len));
        len2 = len;
        apr_err = apr_file_write (fh, buffer, &len2);
        if ((apr_err) || (len2 != len))
            return svn_error_createf 
            (apr_err ? apr_err : SVN_ERR_INCOMPLETE_DATA, NULL,
            "Error writing contents of '%s'", path);
        if (len != sizeof (buffer))
            break;
    }

    /* And close the file. */
    apr_file_close (fh);
    return SVN_NO_ERROR;
}

static svn_error_t *
open_writable_binary_file (apr_file_t **fh, 
                           const char *path /* UTF-8! */, 
                           apr_pool_t *pool)
{
    apr_array_header_t *path_pieces;
    svn_error_t *err;
    int i;
    const char *full_path, *dir;

    /* Try the easy way to open the file. */
    err = svn_io_file_open (fh, path, 
        APR_WRITE | APR_CREATE | APR_TRUNCATE | APR_BINARY,
        APR_OS_DEFAULT, pool);
    if (! err)
        return SVN_NO_ERROR;

    svn_path_split (path, &dir, NULL, pool);

    /* If the file path has no parent, then we've already tried to open
    it as best as we care to try above. */
    if (svn_path_is_empty (dir))
        return svn_error_createf (err->apr_err, err,
        "Error opening writable file '%s'", path);

    path_pieces = svn_path_decompose (dir, pool);
    if (! path_pieces->nelts)
        return SVN_NO_ERROR;

    full_path = "";
    for (i = 0; i < path_pieces->nelts; i++)
    {
        svn_node_kind_t kind;
        const char *piece = ((const char **) (path_pieces->elts))[i];
        full_path = svn_path_join (full_path, piece, pool);
        SVN_ERR (svn_io_check_path (full_path, &kind, pool));

        /* Does this path component exist at all? */
        if (kind == svn_node_none)
        {
            SVN_ERR (svn_io_dir_make (full_path, APR_OS_DEFAULT, pool));
        }
        else if (kind != svn_node_dir)
        {
            if (err)
                return svn_error_createf (err->apr_err, err,
                "Error creating dir '%s' (path exists)", 
                full_path);
        }
    }

    /* Now that we are ensured that the parent path for this file
    exists, try once more to open it. */
    err = svn_io_file_open (fh, path, 
        APR_WRITE | APR_CREATE | APR_TRUNCATE | APR_BINARY,
        APR_OS_DEFAULT, pool);
    if (err)
        return svn_error_createf (err->apr_err, err,
        "Error opening writable file '%s'", path);

    return SVN_NO_ERROR;
}

/* Prepare temporary files *TMPFILE1 and *TMPFILE2 for diffing
PATH1@ROOT1 versus PATH2@ROOT2.  If either ROOT1 or ROOT2 is NULL,
the temporary file for its path/root will be an empty one.
Otherwise, its temporary file will contain the contents of that
path/root in the repository.

An exception to this is when either path/root has an svn:mime-type
property set on it which indicates that the file contains
non-textual data -- in this case, the *IS_BINARY flag is set and no
temporary files are created.

Use POOL for all that allocation goodness. */
static svn_error_t *
prepare_tmpfiles (const char **tmpfile1,
                  const char **tmpfile2,
                  svn_boolean_t *is_binary,
                  svn_fs_root_t *root1,
                  const char *path1,
                  svn_fs_root_t *root2,
                  const char *path2,
                  apr_pool_t *pool)
{
    svn_string_t *mimetype;
    apr_file_t *fh;

    /* Init the return values. */
    *tmpfile1 = NULL;
    *tmpfile2 = NULL;
    *is_binary = FALSE;

    /* Check for binary mimetypes.  If either file has a binary
    mimetype, get outta here.  */
    if (root1)
    {
        SVN_ERR (svn_fs_node_prop (&mimetype, root1, path1, 
            SVN_PROP_MIME_TYPE, pool));
        if (mimetype && svn_mime_type_is_binary (mimetype->data))
        {
            *is_binary = TRUE;
            return SVN_NO_ERROR;
        }
    }
    if (root2)
    {
        SVN_ERR (svn_fs_node_prop (&mimetype, root2, path2, 
            SVN_PROP_MIME_TYPE, pool));
        if (mimetype && svn_mime_type_is_binary (mimetype->data))
        {
            *is_binary = TRUE;
            return SVN_NO_ERROR;
        }
    }

    /* Now, prepare the two temporary files, each of which will either
    be empty, or will have real contents.  The first file we will
    make in our temporary directory. */
    *tmpfile2 = svn_path_join (".rssfeed", path2, pool);
    SVN_ERR (open_writable_binary_file (&fh, *tmpfile2, pool));
    if (root2)
        SVN_ERR (dump_contents (fh, root2, path2, pool));
    apr_file_close (fh);

    /* The second file is constructed from the first one's path. */
    SVN_ERR (svn_io_open_unique_file (&fh, tmpfile1, *tmpfile2, 
        NULL, FALSE, pool));
    if (root1)
        SVN_ERR (dump_contents (fh, root1, path1, pool));
    apr_file_close (fh);

    return SVN_NO_ERROR;
}

/* Recursively print all nodes in the tree that have been modified
(do not include directories affected only by "bubble-up"). */
static svn_error_t *
print_diff_tree (svn_fs_root_t *root,
                 svn_fs_root_t *base_root,
                 svn_repos_node_t *node, 
                 const char *path /* UTF-8! */,
                 const char *base_path /* UTF-8! */,
                 RSSFeed_ctxt_t *ctx,
                 apr_file_t * out,
                 apr_pool_t *pool)
{
    const char *orig_path = NULL, *new_path = NULL, *path_native;
    svn_boolean_t do_diff = FALSE;
    svn_boolean_t is_copy = FALSE;
    svn_boolean_t binary = FALSE;

    if (! node)
        return SVN_NO_ERROR;

    SVN_ERR (svn_utf_cstring_from_utf8 (&path_native, path, pool));

    /* Print copyfrom history for the top node of a copied tree. */
    if ((SVN_IS_VALID_REVNUM (node->copyfrom_rev))
        && (node->copyfrom_path != NULL))
    {
        const char *base_path_native;

        /* This is ... a copy. */
        is_copy = TRUE;

        /* Propagate the new base.  Copyfrom paths usually start with a
        slash; we remove it for consistency with the target path.
        ### Yes, it would be *much* better for something in the path
        library to be taking care of this! */
        if (node->copyfrom_path[0] == '/')
            base_path = apr_pstrdup (pool, node->copyfrom_path + 1);
        else
            base_path = apr_pstrdup (pool, node->copyfrom_path);

        SVN_ERR (svn_utf_cstring_from_utf8 (&base_path_native, base_path, pool));

        apr_file_printf (out, "Copied: %s (from rev %" SVN_REVNUM_T_FMT ", %s)\n",
            path_native, node->copyfrom_rev, base_path_native);

        SVN_ERR (svn_fs_revision_root (&base_root,
            svn_fs_root_fs (base_root),
            node->copyfrom_rev, pool));
    }

    /* First, we'll just print file content diffs. */
    if (node->kind == svn_node_file)
    {
        /* Here's the generalized way we do our diffs:

        - First, we'll check for svn:mime-type properties on the old
        and new files.  If either has such a property, and it
        represents a binary type, we won't actually be doing a real
        diff.

        - First, dump the contents of the new version of the file
        into the svnlook temporary directory, building out the
        actual directories that need to be created in order to
        fully represent the filesystem path inside the tmp
        directory.

        - Then, dump the contents of the old version of the file into
        the top level of the svnlook temporary directory using a
        unique temporary file name (we do this *after* putting the
        new version of the file there in case something actually
        versioned has a name that looks like one of our unique
        identifiers).

        - Next, we run 'diff', passing the repository path as the
        label.  

        - Finally, we delete the temporary files (but leave the
        built-out directories in place until after all diff
        handling has been finished).  */
        if ((node->action == 'R') && (node->text_mod))
        {
            do_diff = TRUE;
            SVN_ERR (prepare_tmpfiles (&orig_path, &new_path, &binary,
                base_root, base_path, root, path, pool));
        }
        if ((node->action == 'A') && (node->text_mod))
        {
            do_diff = TRUE;
            SVN_ERR (prepare_tmpfiles (&orig_path, &new_path, &binary,
                NULL, base_path, root, path, pool));
        }
        if (node->action == 'D')
        {
            do_diff = TRUE;
            SVN_ERR (prepare_tmpfiles (&orig_path, &new_path, &binary,
                base_root, base_path, NULL, path, pool));
        }
    }

    if (do_diff)
    {
        if (! is_copy)
            apr_file_printf (out, "%s: %s\n", 
            ((node->action == 'A') ? "Added" : 
        ((node->action == 'D') ? "Deleted" :
        ((node->action == 'R') ? "Modified" : "Index"))),
            path_native);

        if (node->action != 'D')
        {
            svn_diff_t *diff;

            apr_file_printf (out, "===========================================================\
===================\n");

            if (binary)
            {
                apr_file_printf (out, "(Binary files differ)\n");
            }
            else
            {
                SVN_ERR (svn_diff_file_diff (&diff, orig_path, new_path, pool));
                if (svn_diff_contains_diffs (diff))
                {
                    const char *orig_label, *new_label;

                    SVN_ERR (generate_label (&orig_label, base_root, 
                        base_path, pool));
                    SVN_ERR (generate_label (&new_label, root, path, pool));
                    SVN_ERR (svn_diff_file_output_unified (out, diff, 
                        orig_path, new_path,
                        orig_label, new_label,
                        pool));
                }
            }
        }

        apr_file_printf(out, "\n");
    }
    else if (is_copy)
    {
        apr_file_printf (out, "\n");
    }

    /* Now, delete any temporary files. */
    if (orig_path)
        svn_io_remove_file (orig_path, pool);
    if (new_path)
        svn_io_remove_file (new_path, pool);

    /* Return here if the node has no children. */
    node = node->child;
    if (! node)
        return SVN_NO_ERROR;

    /* Handle children and siblings. */
    {
        apr_pool_t *subpool = svn_pool_create (pool);

        /* Recurse down into children. */
        SVN_ERR (print_diff_tree
            (root, base_root, node,
            svn_path_join (path, node->name, subpool),
            svn_path_join (base_path, node->name, subpool),
            ctx, out,
            subpool));

        /* Recurse across siblings. */
        while (node->sibling)
        {
            node = node->sibling;

            SVN_ERR (print_diff_tree
                (root, base_root, node,
                svn_path_join (path, node->name, subpool),
                svn_path_join (base_path, node->name, subpool),
                ctx, out, 
                pool));
        }

        apr_pool_destroy (subpool);
    }

    return SVN_NO_ERROR;
}

/* Generate a generic delta tree. */
static svn_error_t *
generate_delta_tree (svn_repos_node_t **tree,
                     svn_repos_t *repos,
                     svn_fs_root_t *root, 
                     svn_revnum_t base_rev, 
                     svn_boolean_t use_copy_history,
                     apr_pool_t *pool)
{
    svn_fs_root_t *base_root;
    const svn_delta_editor_t *editor;
    void *edit_baton;
    apr_pool_t *edit_pool = svn_pool_create (pool);
    svn_fs_t *fs = svn_repos_fs (repos);

    /* Get the base root. */
    SVN_ERR (svn_fs_revision_root (&base_root, fs, base_rev, pool));

    /* Request our editor. */
    SVN_ERR (svn_repos_node_editor (&editor, &edit_baton, repos,
        base_root, root, pool, edit_pool));

    /* Drive our editor. */
    SVN_ERR (svn_repos_replay (root, editor, edit_baton, edit_pool));

    /* Return the tree we just built. */
    *tree = svn_repos_node_from_baton (edit_baton);
    svn_pool_destroy (edit_pool);
    return SVN_NO_ERROR;
}

static svn_error_t *
get_root (svn_fs_root_t **root,
          RSSFeed_ctxt_t *c,
          apr_pool_t *pool)
{
    SVN_ERR (svn_fs_revision_root (root, c->fs, c->rev_num, pool));
    return SVN_NO_ERROR;
}

/* Print some diff-y stuff in a TBD way. :-) */
static svn_error_t *
do_diff (RSSFeed_ctxt_t *c, apr_file_t *out, apr_pool_t *pool)
{
    svn_fs_root_t *root, *base_root;
    svn_revnum_t base_rev_num;
    svn_repos_node_t *tree;

    SVN_ERR (get_root (&root, c, pool));
    base_rev_num = c->rev_num - 1;

    if (! SVN_IS_VALID_REVNUM (base_rev_num))
        return svn_error_createf 
        (SVN_ERR_FS_NO_SUCH_REVISION, NULL,
        "Transaction '%s' is not based on a revision.  How odd.",
        c->txn_name);

    SVN_ERR (generate_delta_tree (&tree, c->repos, root, base_rev_num, 
        TRUE, pool)); 
    if (tree)
    {
        svn_node_kind_t kind;

        SVN_ERR (svn_fs_revision_root (&base_root, c->fs, base_rev_num, pool));
        SVN_ERR (print_diff_tree (root, base_root, tree, "", "",
            c, out, pool));
        SVN_ERR (svn_io_check_path (".rssfeed", &kind, pool));
        if (kind == svn_node_dir)
            SVN_ERR (svn_io_remove_dir (".rssfeed", pool));
    }
    return SVN_NO_ERROR;
}
svn_boolean_t write_string_to_file(apr_file_t * file, const char * string)
{
    apr_size_t len = strlen(string);
    apr_size_t lenwritten = len;
    apr_file_write (file, string, &lenwritten);
    return len==lenwritten;
}
int main(int argc, char* argv[])
{
    apr_pool_t *pool;
    RSSFeed_ctxt_t ctx;
    RSSFeed_config_t rssdata;
    svn_config_t * cfg;
    const char * cfg_temp_data;
    apr_file_t * rssfile;
    apr_file_t * difffile;

    /* Initialize the app. */
    if (svn_cmdline_init ("RSSFeed", stderr) != EXIT_SUCCESS)
        return EXIT_FAILURE;
    /* Create our top-level pool. */
    pool = svn_pool_create (NULL);

    if (argc <= 3)
    {
        printf("RSSFeed Path_to_Repository RevisionNumber Path_to_ConfigFile\n");
        svn_pool_destroy (pool);
        return EXIT_FAILURE;
    }

    const char *arg = NULL;
    /* Get the repository. */
    SVN_INT_ERR (svn_utf_cstring_to_utf8 (&ctx.repos_path,
        argv[1],
        NULL, pool));
    ctx.repos_path = svn_path_internal_style (ctx.repos_path, pool);

    /* Get revision number */
    SVN_INT_ERR (svn_utf_cstring_to_utf8 (&arg, argv[2], NULL, pool));
    ctx.rev_num = SVN_STR_TO_REV (arg);

    /* get path to the config file */
    SVN_INT_ERR (svn_utf_cstring_to_utf8 (&ctx.config_path,
        argv[3],
        NULL, pool));
    ctx.config_path = svn_path_internal_style (ctx.config_path, pool);
    SVN_INT_ERR (svn_config_read (&cfg, ctx.config_path, false, pool));

    svn_config_get (cfg, &cfg_temp_data, "rss", "location", "/");
    rssdata.location = apr_pstrdup(pool, cfg_temp_data);

    svn_config_get (cfg, &cfg_temp_data, "rss", "title", "Subversion Repository");
    rssdata.title = apr_pstrdup(pool, cfg_temp_data);

    svn_config_get (cfg, &cfg_temp_data, "rss", "link", "http://localhost");
    rssdata.link = apr_pstrdup(pool, cfg_temp_data);

    svn_config_get (cfg, &cfg_temp_data, "rss", "description", "commit logs of the subversion project");
    rssdata.description = apr_pstrdup(pool, cfg_temp_data);

    svn_config_get (cfg, &cfg_temp_data, "rss", "ttl", "30");
    rssdata.ttl = apr_pstrdup(pool, cfg_temp_data);

    svn_config_get (cfg, &cfg_temp_data, "rss", "imageurl", "");
    rssdata.imageurl = apr_pstrdup(pool, cfg_temp_data);

    svn_config_get (cfg, &cfg_temp_data, "rss", "items", "15");
    rssdata.items = SVN_STR_TO_REV(cfg_temp_data);

    svn_config_get (cfg, &cfg_temp_data, "rss", "maxsize", "5000");
    rssdata.maxsize = SVN_STR_TO_REV(cfg_temp_data);

    svn_config_get (cfg, &cfg_temp_data, "rss", "difffolder", ".");
    rssdata.difffolder = apr_pstrdup(pool, cfg_temp_data);

    svn_config_get (cfg, &cfg_temp_data, "rss", "difffolderurl", "http://localhost");
    rssdata.difffolderurl = apr_pstrdup(pool, cfg_temp_data);

    ctx.outputbuffer = (char*)apr_pcalloc (pool, rssdata.maxsize);
    ctx.outputbufferlength = rssdata.maxsize;

    /* open the repository */
    SVN_INT_ERR (svn_repos_open (&(ctx.repos), ctx.repos_path, pool));
    ctx.fs = svn_repos_fs (ctx.repos);

    /* create the ouput file */
    SVN_INT_ERR (svn_io_file_open (&rssfile, rssdata.location, 
        APR_WRITE | APR_CREATE | APR_TRUNCATE | APR_BINARY,
        APR_OS_DEFAULT, pool));

    write_string_to_file(rssfile, "<?xml version=\"1.0\" ?>");
    write_string_to_file(rssfile, "<!--  RSS generated by Radio UserLand v8.0.5 on 9/30/2002; 4:00:00 AM Pacific -->");
    write_string_to_file(rssfile, "<rss version=\"2.0\" xmlns:blogChannel=\"");
    write_string_to_file(rssfile, rssdata.link);
    write_string_to_file(rssfile, "\">");
    write_string_to_file(rssfile, "<channel>");
    write_string_to_file(rssfile, "<title>");
    write_string_to_file(rssfile, rssdata.title);
    write_string_to_file(rssfile, "</title>");
    write_string_to_file(rssfile, "<link>");
    write_string_to_file(rssfile, rssdata.link);
    write_string_to_file(rssfile, "</link>");
    write_string_to_file(rssfile, "<description>");
    write_string_to_file(rssfile, rssdata.description);
    write_string_to_file(rssfile, "</description>");
    write_string_to_file(rssfile, "<ttl>");
    write_string_to_file(rssfile, rssdata.ttl);
    write_string_to_file(rssfile, "</ttl>");
    write_string_to_file(rssfile, "<image>");
    write_string_to_file(rssfile, "<title>");
    write_string_to_file(rssfile, rssdata.title);
    write_string_to_file(rssfile, "</title>");
    write_string_to_file(rssfile, "<url>");
    write_string_to_file(rssfile, rssdata.imageurl);
    write_string_to_file(rssfile, "</url>");
    write_string_to_file(rssfile, "<link>");
    write_string_to_file(rssfile, rssdata.link);
    write_string_to_file(rssfile, "</link>");
    write_string_to_file(rssfile, "</image>");

    for (int loop = 0; loop < rssdata.items; loop++)
    {
        write_string_to_file(rssfile, "<item>");
        write_string_to_file(rssfile, "<title>");

        /* get the author */
        svn_string_t *prop_value_author;
        SVN_INT_ERR (get_property (&prop_value_author, &ctx, SVN_PROP_REVISION_AUTHOR, pool));
        if (prop_value_author && prop_value_author->data)
            //strncat(ctx.outputbuffer, prop_value_author->data, ctx.outputbufferlength);
            write_string_to_file(rssfile, prop_value_author->data);
        write_string_to_file(rssfile, ", ");

        /* get the date */
        svn_string_t *prop_value_date;

        SVN_INT_ERR (get_property (&prop_value_date, &ctx, SVN_PROP_REVISION_DATE, pool));
        if (prop_value_date && prop_value_date->data)
        {
            /* Convert the date for humans. */
            apr_time_t aprtime;

            SVN_INT_ERR (svn_time_from_cstring (&aprtime, prop_value_date->data, pool));
            write_string_to_file(rssfile, svn_time_to_human_cstring (aprtime, pool));
        }
        write_string_to_file(rssfile, ", revision: ");
        write_string_to_file(rssfile, argv[2]);
        write_string_to_file(rssfile, "</title>");

        write_string_to_file(rssfile, "<description>");

        /* get the log */
        svn_string_t *prop_value_log;

        SVN_INT_ERR (get_property (&prop_value_log, &ctx, SVN_PROP_REVISION_LOG, pool));
        if (! (prop_value_log && prop_value_log->data))
        {
            strncat (ctx.outputbuffer, "\n", ctx.outputbufferlength);
        }
        strncat(ctx.outputbuffer, prop_value_log->data, ctx.outputbufferlength);


        write_string_to_file(rssfile, ctx.outputbuffer);
        write_string_to_file(rssfile, "</description>");
        write_string_to_file(rssfile, "<link>");
        apr_file_printf(rssfile, "%s/%d.txt", rssdata.difffolderurl, loop);
        write_string_to_file(rssfile, "</link>");
        write_string_to_file(rssfile, "</item>");

        char buf[8096];
        sprintf(buf, "%s/%d.txt", rssdata.difffolder, loop);
        /* create the diff file */
        SVN_INT_ERR (svn_io_file_open (&difffile, buf, 
            APR_WRITE | APR_CREATE | APR_TRUNCATE | APR_BINARY,
            APR_OS_DEFAULT, pool));

        do_diff(&ctx, difffile, pool);

        apr_file_close(difffile);
        ctx.rev_num--;
        if (ctx.rev_num < 1)
            break;
        *ctx.outputbuffer = '\0';
    }
    write_string_to_file(rssfile, "</channel>");
    write_string_to_file(rssfile, "</rss>");
    apr_file_close (rssfile);

    return 0;
}

